%%% A simple decision support system with rule for CSP handling

run:-
     write('Podaj ogran.'),
     read(Ogr),
     generate(1,[],Result),
     findcost(Result,Cost),
     Cost < Ogr,
     write(Result),write('     '),write(Cost),nl,
     fail.
run.     
generate(N,L,Result):-
     choice_set(N,List,unique), !,
     select(El,List,_),
     N1 is N + 1,
     generate(N1,[El|L],Result).
generate(N,List,Result):-
     choice_set(N,Set,subset), !,
     subset(Subset,Set),
     N1 is N + 1,
     append(Subset,List,NewList),
     generate(N1,NewList,Result).
generate(_,Result,Result):-
     test(1,Result),!.

test(N,_):-
     \+(rule(N,_,_)).
test(N,Result):-
     rule(N,Preclist,Conclist),
     satisfied(Preclist,Result),!,
     satisfied(Conclist,Result),
     N1 is N + 1,
     test(N1,Result).     
test(N,Result):-
     N1 is N + 1,!,
     test(N1,Result).

satisfied([],_):-!.
satisfied([selected(X)|Rest],Result):-
     member(X,Result),!,
     satisfied(Rest,Result).
satisfied([unselected(X)|Rest],Result):-
     \+(member(X,Result)),!,
     satisfied(Rest,Result).     
satisfied([impossible|_],_):-!,
     fail.

findcost([],0):-!.
findcost([X|L],Cost):-
     cost(X,C),
     findcost(L,CostL),
     Cost is C + CostL.


%%% subset([],_).
%%% subset([El|List],Set) :- select(El,Set,Rest), subset(List,Rest).

subset([],[]):- !.
subset([G|T],[G|TT]):- subset(T,TT).
subset(T,[_|TT]):- subset(T,TT).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%
%%% Problem definition
%%%
%%% t - technologies
%%% e - energy supply system
%%% s - environment protection
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

choice_set(1,[t(1),t(2),t(3)],unique).
choice_set(2,[e(1),e(2),e(3),e(4)],unique).
choice_set(3,[s(1),s(2),s(3),s(4),s(5)],subset).

cost(t(1),90). cost(t(2),110). cost(t(3),140).
cost(e(1),20). cost(e(2),40). cost(e(3),60). cost(e(4),80).
cost(s(1),10). cost(s(2),20). cost(s(3),30). cost(s(4),40). cost(s(5),50).

rule(1,[selected(t(1))],[selected(s(1)),
                         selected(s(2)),
                         selected(s(3))]).
rule(2,[selected(t(2))],[selected(s(2)),
                         selected(s(3))]).
rule(3,[selected(t(3))],[selected(s(3))]).
rule(4,[selected(e(1))],[selected(s(3)),
                         selected(s(4)),
                         selected(s(5))]).
rule(5,[selected(e(2))],[selected(s(4)),
                         selected(s(5))]).
rule(6,[selected(e(3))],[selected(s(5))]).
rule(7,[selected(t(1))],[unselected(e(3)),
                         unselected(e(4))]).
rule(8,[selected(t(2))],[unselected(e(4))]).
rule(9,[],[selected(t(_))]).
rule(10,[],[selected(e(_))]).
rule(11,[selected(X),unselected(X)],[impossible]).





